@@ -1,7 +1,8 @@
 from pathlib import Path
+from telnetlib import IP
+from typing import Callable
 
-from pinjected import Design, instances, injected_function, Injected
-import openai
+from pinjected import injected,Design,instances,IProxy
 
 __meta_design__: Design = instances(
     default_design_paths=["pinjected.demo.default_design"]
@@ -15,18 +16,19 @@ default_design: Design = instances(
 )
 
 
-@injected_function
+@injected
 def LLM(load_openai_api_key: str, model, max_tokens, /, prompt) -> str:
     # openai_api_key = Path("~/openai_api_key.txt").expanduser().read_text().strip()
+    import openai
     return openai.Completion.create(
         load_openai_api_key(), prompt=prompt, model=model, max_tokens=max_tokens
     )["choices"][0]["text"]
 
 
-@injected_function
-def Gandalf(LLM: "str -> str", /, user_message: str) -> str:
+@injected
+def Gandalf(LLM: Callable[[str], str], /, user_message: str) -> str:
     return LLM(f"Respond to the user's message as if you were a Gandalf: {user_message}")
 
 
-test_greeting: Injected = Gandalf("How are you?")
+test_greeting: IProxy = Gandalf("How are you?")
 
