@@ -3,8 +3,8 @@ from typing import Set, Awaitable, TypeVar, Callable
 
 from pinjected import Injected
 from pinjected.di.applicative import Applicative
-from pinjected.di.ast import Expr, Object, show_expr, UnaryOp, Call, BiOp, Attr, GetItem
-from pinjected.di.injected import InjectedPure, InjectedFunction, InjectedByName
+from pinjected.di.expr_util import Expr, Object, show_expr, UnaryOp, Call, BiOp, Attr, GetItem
+from pinjected.di.injected import InjectedPure, InjectedFromFunction, InjectedByName
 from pinjected.di.proxiable import T, DelegatedVar
 from pinjected.di.static_proxy import eval_applicative, ast_proxy, \
     AstProxyContextImpl
@@ -28,7 +28,7 @@ class ApplicativeInjectedImpl(Applicative[Injected]):
 
     def _await_(self, tgt: Injected):
         async def awaiter(x):
-            # from loguru import logger
+            # from pinjected.logging import logger
             # logger.info(f"awaiting {x} due to await UnaryOp")
             res = await x
             # logger.info(f"obtained {res} for UnaryOp")
@@ -36,6 +36,48 @@ class ApplicativeInjectedImpl(Applicative[Injected]):
 
         return tgt.map(awaiter)
 
+    def unary(self, op: str, tgt: T):
+        async def unary_op(x):
+            if op == '-':
+                return -x
+            if op == '~':
+                return ~x
+            if op == 'len':
+                return len(x)
+            raise NotImplementedError(f"unary op {op} not implemented")
+
+        return tgt.map(unary_op)
+
+    def biop(self, op: str, tgt: T, other):
+        async def bi_op(x, y):
+            if op == '+':
+                return x + y
+            if op == '-':
+                return x - y
+            if op == '*':
+                return x * y
+            if op == '/':
+                return x / y
+            if op == '%':
+                return x % y
+            if op == '**':
+                return x ** y
+            if op == '<<':
+                return x << y
+            if op == '>>':
+                return x >> y
+            if op == '&':
+                return x & y
+            if op == '^':
+                return x ^ y
+            if op == '|':
+                return x | y
+            if op == '//':
+                return x // y
+            raise NotImplementedError(f"bi op {op} not implemented")
+
+        return tgt.map(lambda x: other.map(lambda y: bi_op(x, y)))
+
 
 @dataclass
 class EvaledInjected(Injected[T]):
@@ -45,6 +87,7 @@ class EvaledInjected(Injected[T]):
 
     def __post_init__(self):
         assert isinstance(self.ast, Expr)
+        self.__expr__ = self.ast
 
     def dependencies(self) -> Set[str]:
         return self.value.dependencies()
@@ -53,7 +96,8 @@ class EvaledInjected(Injected[T]):
         return self.value.get_provider()
 
     def __str__(self):
-        return f"EvaledInjected(value={self.value},ast={show_expr(self.ast, reduce_injected_expr)})"
+        #return f"EvaledInjected(value={self.value},ast={show_expr(self.ast, reduce_injected_expr)})"
+        return f"Eval({show_expr(self.ast)})"
 
     def __repr__(self):
         return str(self)
@@ -67,12 +111,16 @@ class EvaledInjected(Injected[T]):
     def dynamic_dependencies(self) -> Set[str]:
         return self.value.dynamic_dependencies()
 
+    def __repr_expr__(self):
+        return show_expr(self.ast)
+
+
 
 def reduce_injected_expr(expr: Expr):
     match expr:
         case Object(InjectedPure(value)):
             return str(value)
-        case Object(InjectedFunction(func, kwargs)):
+        case Object(InjectedFromFunction(func, kwargs)):
             reduced = reduce_injected_expr(Object(kwargs))
             return f"{func.__name__}({reduced})"
         case Object(DelegatedVar() as dv):
@@ -114,30 +162,40 @@ def eval_injected(expr: Expr[Injected]) -> EvaledInjected:
 
 
 def walk_replace(expr: Expr, transformer: Callable[[Expr], Expr]):
-    match expr:
-        case Object(x):
-            return transformer(Object(x))
-        case Call(f, args, kwargs):
-            return transformer(
-                Call(f,
-                     tuple([walk_replace(a, transformer) for a in args]),
-                     {k: walk_replace(v, transformer) for k, v in kwargs.items()}
-                     )
-            )
-        case BiOp(op, left, right):
-            return transformer(BiOp(op, walk_replace(left, transformer), walk_replace(right, transformer)))
-        case UnaryOp(op, tgt):
-            return transformer(UnaryOp(op, walk_replace(tgt, transformer)))
-        case Attr(data, name):
-            return transformer(Attr(walk_replace(data, transformer), name))
-        case GetItem(data, key):
-            return transformer(GetItem(walk_replace(data, transformer), walk_replace(key, transformer)))
-        case _:
-            return expr
+    memo = dict()
+    from pinjected.pinjected_logging import logger
+
+    def impl(expr):
+        match expr:
+            case Object(DelegatedVar(Expr() as nested_expr, cxt) as dv):
+                res = impl(nested_expr)
+            case Object(x):
+                res = transformer(Object(x))
+            case Call(f, args, kwargs):
+                res = transformer(
+                    Call(
+                        impl(f),
+                        tuple([impl(a) for a in args]),
+                        {k: impl(v) for k, v in kwargs.items()}
+                    )
+                )
+            case BiOp(op, left, right):
+                res = transformer(BiOp(op, impl(left), impl(right)))
+            case UnaryOp(op, tgt):
+                res = transformer(UnaryOp(op, impl(tgt)))
+            case Attr(data, name):
+                res = transformer(Attr(impl(data), name))
+            case GetItem(data, key):
+                res = transformer(GetItem(impl(data), impl(key)))
+            case _:
+                res = expr
+        return res
+
+    return impl(expr)
 
 
 def await_awaitables(expr: Expr[T]) -> Expr:
-    from loguru import logger
+    from pinjected.pinjected_logging import logger
     # logger.info(f"await_awaitables {expr}")
 
     def transformer(expr: Expr):
@@ -146,12 +204,16 @@ def await_awaitables(expr: Expr[T]) -> Expr:
                 return UnaryOp('await', expr)
             case Call(Object(object(__is_async_function__=True)), args, kwargs) as call:
                 return UnaryOp('await', call)
+            case Call(object(__is_async_function__=True), args, kwargs) as call:
+                return UnaryOp('await', call)
             case _:
                 return expr
 
     return walk_replace(expr, transformer)
 
 
+
+
 def injected_proxy(injected: Injected) -> DelegatedVar[Injected]:
     return ast_proxy(Object(injected), InjectedEvalContext)
 
