@@ -25,7 +25,7 @@ class Applicative(Generic[T], ABC):
         items = list(kwargs.items())
         keys = [t[0] for t in items]
         values = [t[1] for t in items]
-        from loguru import logger
+        from pinjected.pinjected_logging import logger
         # logger.info(f"keys:{keys}")
         # logger.info(f"values:{values}")
         def mapper(vs):
@@ -37,3 +37,9 @@ class Applicative(Generic[T], ABC):
 
     def _await_(self, tgt: T):
         raise NotImplementedError("await not implemented")
+
+    def unary(self, op: str, tgt: T):
+        raise NotImplementedError("unary not implemented")
+
+    def biop(self, op: str, tgt: T, other):
+        raise NotImplementedError("biop not implemented")
