@@ -1,34 +1,26 @@
 import inspect
-from copy import copy
 from dataclasses import dataclass, field, replace
 from functools import wraps
-from typing import TypeVar, List, Dict, Union, Callable, Type, Self
+from typing import TypeVar, List, Dict, Union, Callable, Type, Optional
 
 from cytoolz import merge
 from makefun import create_function
 
-from pinjected.di.graph import DependencyResolver
+from pinjected.di.design_interface import ProvisionValidator, Design
+from pinjected.v2.callback import IResolverCallback
+from pinjected.di.app_injected import EvaledInjected
 from pinjected.di.implicit_globals import IMPLICIT_BINDINGS
 from pinjected.di.injected import Injected
-from pinjected.di.injected import extract_dependency_including_self, InjectedPure, InjectedFunction
+from pinjected.di.injected import extract_dependency_including_self, InjectedPure, InjectedFromFunction
 # from pinjected.di.util import get_class_aware_args, get_dict_diff, check_picklable
 from pinjected.di.proxiable import DelegatedVar
-from pinjected.v2.binds import IBind, BindInjected
+from pinjected.v2.binds import IBind, BindInjected, ExprBind
 from pinjected.v2.keys import IBindKey, StrBindKey
 
 T = TypeVar("T")
 U = TypeVar("U")
 
 
-def map_wrap(src, f):
-    @wraps(src)
-    def wrapper(*args, **kwargs):
-        return f(src(*args, **kwargs))
-
-    wrapper.__signature__ = inspect.signature(src)
-    return wrapper
-
-
 def remove_kwargs_from_func(f, kwargs: List[str]):
     deps = extract_dependency_including_self(f)
     to_remove = set(kwargs)
@@ -48,7 +40,106 @@ def remove_kwargs_from_func(f, kwargs: List[str]):
 
 
 @dataclass
-class Design:
+class MergedDesign(Design):
+    srcs: List[Design]
+
+    @property
+    def children(self):
+        return self.srcs
+
+    def __contains__(self, item: IBindKey):
+        return any(item in src for src in self.srcs)
+
+    def __getitem__(self, item: IBindKey | str):
+        for src in reversed(self.srcs):  # the last one takes precedence
+            if item in src:
+                res = src[item]
+                assert isinstance(res, IBind), f"item must be IBind, but got {type(res)}=={res}"
+                return res
+        raise KeyError(f"{item} not found in any of the sources")
+
+    @property
+    def bindings(self) -> Dict[IBindKey, IBind]:
+        return merge(*[src.bindings for src in self.srcs])
+
+    @property
+    def validations(self) -> Dict[IBindKey, ProvisionValidator]:
+        return merge(*[src.validations for src in self.srcs])
+
+    def __repr__(self):
+        return f"MergedDesign(srcs={len(self.srcs)})"
+
+    def __add__(self, other):
+        return MergedDesign(self.srcs + [other])
+
+
+@dataclass
+class AddValidation(Design):
+    src: Design
+    _validations: Dict[IBindKey, ProvisionValidator]
+
+    def __contains__(self, item: IBindKey):
+        return item in self.src
+
+    def __getitem__(self, item: IBindKey | str):
+        return self.src[item]
+
+    @property
+    def bindings(self) -> Dict[IBindKey, IBind]:
+        return self.src.bindings
+
+    @property
+    def validations(self) -> Dict[IBindKey, ProvisionValidator]:
+        return self._validations | self.src.validations
+
+    @property
+    def children(self):
+        return [self.src]
+
+
+"""
+All designs are merged with '+'. 
+so, those things added doesnt require any design information...
+So basically the design tree becomes a tree from MergedDesign,
+where the leaves are the actual designs.
+wait, in that case, for the priorities to work, the merged design needs to be fixed.
+
+"""
+
+
+@dataclass
+class MetaDataDesign(Design):
+    def __contains__(self, item: IBindKey):
+        return False
+
+    def __getitem__(self, item: IBindKey | str) -> IBind:
+        raise KeyError(f"no such key {item}")
+
+    @property
+    def bindings(self) -> Dict[IBindKey, IBind]:
+        return dict()
+
+    @property
+    def validations(self) -> Dict[IBindKey, ProvisionValidator]:
+        return dict()
+
+    @property
+    def children(self):
+        return []
+
+
+@dataclass
+class AddSummary(MetaDataDesign):
+    summary: str
+
+
+@dataclass
+class AddTags(MetaDataDesign):
+    tags: List[str]
+
+
+@dataclass
+class DesignImpl(Design):
     """
     The ``Design`` class acts as a central registry for managing and applying dependency injection
     within your application. It allows for the binding of various components, instances, and providers,
@@ -84,7 +175,7 @@ class Design:
             def run(self):
                 print(self.dep.a + self.dep.b + self.dep.c + self.dep.d)
 
-        d = Design().bind_instance(
+        d = EmptyDesign.bind_instance(
             a=0,
             b=1
         ).bind_provider(
@@ -110,13 +201,13 @@ class Design:
 
     .. code-block:: python
 
-        d1 = Design().bind_instance(
+        d1 = EmptyDesign.bind_instance(
             a=0
         )
-        d2 = Design().bind_instance(
+        d2 = EmptyDesign.bind_instance(
             b=1
         )
-        d3 = Design().bind_instance(
+        d3 = EmptyDesign.bind_instance(
             b=0
         )
 
@@ -128,34 +219,31 @@ class Design:
     This feature ensures that ``Design`` instances are composable and adaptable, providing
     a robust foundation for building complex, modular applications with dependency injection.
     """
-    bindings: Dict[IBindKey, IBind] = field(default_factory=dict)
-    modules: list = field(default_factory=list)
+
+    _bindings: Dict[IBindKey, IBind] = field(default_factory=dict)
+
+    @property
+    def children(self):
+        return []
+
+    @property
+    def validations(self) -> Dict[IBindKey, ProvisionValidator]:
+        return dict()
+
+    @property
+    def bindings(self):
+        return self._bindings
 
     def __getstate__(self):
         res = dict(
-            bindings=self.bindings,
-            modules=[m.__name__ for m in self.modules],
+            _bindings=self._bindings,
         )
         return res
 
     def __setstate__(self, state):
-        mods = state["modules"]
-        import importlib
-        mods = [importlib.import_module(
-            name=m
-        ) for m in mods]
-        state["modules"] = mods
         for k, v in state.items():
             setattr(self, k, v)
 
-    def __add__(self, other: "Design"):
-        assert isinstance(other, Design), f"cannot add {type(other)} to Design"
-        res = Design(
-            bindings=merge(self.bindings, other.bindings),
-            modules=list(set(self.modules) | set(other.modules)),
-        )
-        return res
-
     def bind_instance(self, **kwargs, ):
         """
         Here, I need to find the CodeLocation for each binding.
@@ -163,60 +251,71 @@ class Design:
         :param __binding_metadata__: a dict of [str,BindMetadata]
         :return:
         """
-        x = self
+        bindings = self._bindings.copy()
         for k, v in kwargs.items():
             if isinstance(v, type):
-                from loguru import logger
+                from pinjected.pinjected_logging import logger
                 logger.warning(f"{k} is bound to class {v} with 'bind_instance' do you mean 'bind_class'?")
-            x += Design({StrBindKey(k): BindInjected(Injected.pure(v))})
-        return x
+            bindings[StrBindKey(k)] = BindInjected(Injected.pure(v))
+        return DesignImpl(_bindings=bindings)
+
+    @staticmethod
+    def to_bind(tgt) -> IBind:
+        from pinjected.pinjected_logging import logger
+        match tgt:
+            case IBind():
+                return tgt
+            case type():
+                return BindInjected(Injected.bind(tgt))
+            case EvaledInjected():
+                return ExprBind(tgt)
+            case DelegatedVar():
+                return ExprBind(tgt.eval())
+            case Injected():
+                return BindInjected(tgt)
+            case non_func if not callable(non_func):
+                logger.warning(f"{tgt}: non-callable or non-injected is passed to bind_provider. fixing automatically.")
+                return BindInjected(Injected.pure(non_func))
+            case func if callable(func):
+                return BindInjected(Injected.bind(func))
+            case _:
+                raise ValueError(f"cannot bind {tgt}")
 
     def bind_provider(self, **kwargs: Union[Callable, Injected]):
-        bindings = dict()
+        bindings = self.bindings.copy()
         for k, v in kwargs.items():
-            # logger.info(f"binding provider:{k}=>{v}")
-            from loguru import logger
-            match v:
-                case type():
-                    #logger.warning(f"{k}->{v}: class is used for bind_provider. fixing automatically.")
-                    bindings[StrBindKey(k)] = BindInjected(Injected.bind(v))
-                case Injected():
-                    bindings[StrBindKey(k)] = BindInjected(v)
-                case DelegatedVar():
-                    bindings[StrBindKey(k)] = BindInjected(v.eval())
-                case non_func if not callable(non_func):
-                    logger.warning(
-                        f"{k}->{v}: non-callable or non-injected is passed to bind_provider. fixing automatically.")
-                    bindings[StrBindKey(k)] = BindInjected(Injected.pure(v))
-                case func if callable(func):
-                    bindings[StrBindKey(k)] = BindInjected(Injected.bind(func))
-                case _:
-                    raise ValueError(f"cannot bind {k} to {v}")
-
-        return self + Design(
-            bindings=bindings,
-        )
+            bindings[StrBindKey(k)] = self.to_bind(v)
+        return DesignImpl(_bindings=bindings)
 
     def add_metadata(self, **kwargs: "BindMetadata") -> "Design":
         res = self
         for k, meta in kwargs.items():
             key = StrBindKey(k)
             bind: IBind = self.bindings[key]
-            res += Design(
-                bindings={key: bind.update_metadata(meta)}
+            res += DesignImpl(
+                _bindings={key: bind.update_metadata(meta)}
             )
         return res
 
-    def to_resolver(self):
-        from pinjected.v2.resolver import AsyncResolver
+    def to_resolver(self, callback: Optional[IResolverCallback] = None):
+        from pinjected.v2.resolver import BaseResolverCallback
+        from pinjected.v2.async_resolver import AsyncResolver
         bindings = {**IMPLICIT_BINDINGS, **self.bindings}
-        return AsyncResolver(Design(bindings=bindings, modules=self.modules))
+        if callback is None:
+            callbacks = []
+        else:
+            assert isinstance(callback, IResolverCallback)
+            callbacks = [callback]
+        return AsyncResolver(
+            DesignImpl(_bindings=bindings),
+            callbacks=callbacks
+        )
 
     def to_graph(self):
         return self.to_resolver().to_blocking()
 
-    def run(self, f, modules=None):
-        return self.to_graph(modules).run(f)
+    def run(self, f):
+        return self.to_graph().run(f)
 
     def provide(self, target: Union[str, Type[T]]) -> T:
         """
@@ -227,9 +326,8 @@ class Design:
         return self.to_resolver().to_blocking().provide(target)
 
     def copy(self):
-        return self.__class__(
-            bindings=self.bindings.copy(),
-            modules=copy(self.modules),
+        return DesignImpl(
+            _bindings=self.bindings.copy(),
         )
 
     def map_value(self, src_key, f):
@@ -239,7 +337,7 @@ class Design:
         :return: Design
         """
         mapped_binding = self.bindings[src_key].map(f)
-        return self + Design({src_key: mapped_binding})
+        return self + DesignImpl({src_key: mapped_binding})
 
     def keys(self):
         return self.bindings.keys()
@@ -249,7 +347,7 @@ class Design:
             copied = self.bindings.copy()
             del copied[key]
             return replace(self,
-                           bindings=copied
+                           _bindings=copied
                            )
         return self
 
@@ -259,7 +357,7 @@ class Design:
     def __getitem__(self, item: IBindKey | str):
         if isinstance(item, str):
             item = StrBindKey(item)
-        assert isinstance(item, IBindKey), f"item must be IBindKey or a str, but got {type(item)}"
+        assert isinstance(item, IBindKey), f"item must be IBindKey, but got {type(item)}=={item}"
         return self.bindings[item]
 
     def __str__(self):
@@ -277,7 +375,7 @@ class Design:
         res = dict()
         for k, v in self.bindings.items():
             match v:
-                case BindInjected(InjectedFunction(f, args)):
+                case BindInjected(InjectedFromFunction(f, args)):
                     res[k] = f.__name__
                 case BindInjected(InjectedPure(value)):
                     res[k] = str(value)
@@ -296,7 +394,7 @@ class Design:
                 method.__name__ = name
                 return method
             except AttributeError as ae:
-                from loguru import logger
+                from pinjected.pinjected_logging import logger
                 logger.warning(f"somehow failed to assign new name to a provider function. trying to wrap.")
 
                 def _wrapper(self, *args, **kwargs):
@@ -306,78 +404,3 @@ class Design:
                 _wrapper.__signature__ = inspect.signature(method)
                 return _wrapper
         return method
-
-    def diff(self, other):
-        from pinjected.di.util import get_dict_diff
-        d = get_dict_diff(self.bindings, other.bindings)
-        return d
-
-    def inspect_picklability(self):
-        from pinjected.di.util import check_picklable
-        from loguru import logger
-        logger.info(f"checking picklability of bindings")
-        check_picklable(self.bindings)
-        logger.info(f"checking picklability of modules")
-        check_picklable(self.modules)
-
-    def add_modules(self, *modules):
-        return self + Design(modules=list(modules))
-
-    def to_vis_graph(self) -> "DIGraph":
-        from pinjected.visualize_di import DIGraph
-        return DIGraph(self)
-
-    def purify(self, target: "Providable"):
-        """
-        given an injected, returns a minimized design which can provide the target.
-        :param target:
-        :return:
-        """
-        # ah sometimes the deps require 'session'
-        # and we don't know if the session has enough bindings to provide the target.
-        resolver = DependencyResolver(self)
-        return resolver.purified_design(target).unbind('__resolver__').unbind('session').unbind('__design__')
-
-
-@dataclass
-class DesignOverrideContext:
-    src: Design
-    callback: Callable[[Self], None]
-    depth: int
-    target_vars: dict = field(default_factory=dict)
-
-    def __enter__(self):
-        frame = inspect.currentframe()
-        parent = frame.f_back
-        # get parent global variables
-        parent_globals = parent.f_globals
-        global_ids = {k: id(v) for k, v in parent_globals.items()}
-        from loguru import logger
-        logger.debug(global_ids)
-        self.last_global_ids = global_ids
-
-    def __exit__(self, exc_type, exc_val, exc_tb):
-        from loguru import logger
-        frame = inspect.currentframe()
-        parent = frame.f_back
-        # get parent global variables
-        parent_globals = parent.f_globals
-        global_ids = {k: id(v) for k, v in parent_globals.items()}
-        changed_keys = []
-        for k in global_ids:
-            if k in self.last_global_ids:
-                if global_ids[k] != self.last_global_ids[k]:
-                    changed_keys.append(k)
-            else:
-                changed_keys.append(k)
-        logger.debug(f"global_ids:{global_ids}")
-        # find instance of DelegatedVar and Injected in the changed globals
-        target_vars = dict()
-        for k in changed_keys:
-            v = parent_globals[k]
-            if isinstance(v, DelegatedVar):
-                target_vars[k] = v
-            if isinstance(v, Injected):
-                target_vars[k] = v
-        self.target_vars = target_vars
-        self.callback(self)
