@@ -16,9 +16,9 @@ from rich.console import Console
 from rich.panel import Panel
 
 from pinjected.di.app_injected import EvaledInjected
-from pinjected.di.ast import Expr
+from pinjected.di.expr_util import Expr
 from pinjected.di.designed import Designed
-from pinjected.di.injected import Injected, InjectedByName, InjectedFunction
+from pinjected.di.injected import Injected, InjectedByName, InjectedFromFunction
 from pinjected.di.metadata.bind_metadata import BindMetadata
 from pinjected.di.metadata.location_data import ModuleVarLocation
 from pinjected.di.proxiable import DelegatedVar
@@ -27,6 +27,7 @@ from pinjected.exceptions import DependencyResolutionFailure, DependencyResoluti
 from pinjected.graph_inspection import DIGraphHelper
 from pinjected.providable import Providable
 from pinjected.v2.binds import IBind
+from pinjected.visualize_di import DIGraph
 
 T = TypeVar("T")
 
@@ -91,7 +92,7 @@ class IObjectGraph(metaclass=ABCMeta):
         designed = self._providable_to_designed(providable)
         item = Sessioned(self, designed)
         ctx = sessioned_ast_context(self)
-        from pinjected.di.ast import Object
+        from pinjected.di.expr_util import Object
         return DelegatedVar(Object(item), ctx)
 
     def sessioned(self, target: Providable) -> DelegatedVar[Union[object, T]]:
@@ -167,7 +168,7 @@ class IScope:
 
     @staticmethod
     def default_trace_logger(event: ProvideEvent):
-        from loguru import logger
+        from pinjected.pinjected_logging import logger
         match event:
             case ProvideEvent(trace, "request", data=None):
                 logger.info(f"{' -> '.join(trace)}")
@@ -354,7 +355,7 @@ class DependencyResolver:
         try:
             deps = self.memoized_deps(tgt, include_dynamic=include_dynamic)
         except NoMappingError as ke:
-            from loguru import logger
+            from pinjected.pinjected_logging import logger
             logger.error(f"failed to find dependency for {tgt} in {' -> '.join(trace)}")
             raise NoMappingError(f"failed to find dependency for {tgt} in {' -> '.join(trace)}") from ke
         yield tgt
@@ -382,7 +383,7 @@ class DependencyResolver:
                 res[d] = self._dependency_tree(d, trace + [d])
             return Success(res)
         except NoMappingError as ke:
-            from loguru import logger
+            from pinjected.pinjected_logging import logger
             # msg = f"failed to find dependency for {tgt} in {' -> '.join(trace)}"
             return Failure(DependencyResolutionFailure(tgt, trace, ke))
 
@@ -433,7 +434,7 @@ class DependencyResolver:
     #     if trace is None:
     #         trace = []
     #     assert isinstance(tgt, str)
-    #     from loguru import logger
+    #     from pinjected.logging import logger
     #
     #     try:
     #         deps = [self._provide(d, scope, trace + [d]) for d in self.memoized_deps(tgt)]
@@ -458,7 +459,7 @@ class DependencyResolver:
 
     def _provide(self, tgt: str, scope: IScope, trace: list[str] = None):
         from collections import deque
-        from loguru import logger
+        from pinjected.pinjected_logging import logger
 
         if trace is None:
             trace = []
@@ -565,20 +566,20 @@ class DependencyResolver:
                 key = Path(ast.origin_frame.filename).name + ":L" + str(ast.origin_frame.lineno) + "#" + str(id(tgt))[
                                                                                                          :6]
                 # key = f"EvaledInjected#{str(id(tgt))}"
-                from loguru import logger
+                from pinjected.pinjected_logging import logger
                 logger.info(f"naming new key: {key} == {original}")
                 res = provide_injected(e, key)
-            case InjectedFunction(func, kwargs) as IF if IF.origin_frame is not None:
+            case InjectedFromFunction(func, kwargs) as IF if IF.origin_frame is not None:
                 frame = IF.origin_frame
                 original = frame.filename + ":" + str(frame.lineno)
                 key = f"InjectedFunction#{str(id(tgt))}"
-                from loguru import logger
+                from pinjected.pinjected_logging import logger
                 logger.info(f"naming new key: {key} == {original}")
                 res = provide_injected(IF, key)
             case DelegatedVar(value, cxt) as dv:
                 res = self.provide(dv.eval(), scope)
             case Injected():
-                from loguru import logger
+                from pinjected.pinjected_logging import logger
                 logger.info(f"default injected type:{type(tgt)}")
                 provider = tgt.get_provider()
                 key = provider.__name__ + "#" + str(id(tgt))
@@ -592,9 +593,10 @@ class DependencyResolver:
 
     def child(self, session_provider, overrides: 'Design' = None):
         if overrides is None:
-            from pinjected import Design
-            overrides = Design()
-        child_design = self.src + overrides.bind_provider(
+            from pinjected import Design,EmptyDesign
+            overrides = EmptyDesign()
+        from pinjected import providers
+        child_design = self.src + providers(
             session=session_provider
         )
         child_resolver = DependencyResolver(child_design)
@@ -607,7 +609,7 @@ def run_coroutine_in_new_thread(coroutine):
 
     # Function to run the coroutine in a new event loop
     def run_coroutine():
-        from loguru import logger
+        from pinjected.pinjected_logging import logger
         logger.info(f"running coroutine in new thread")
         loop = asyncio.new_event_loop()
         asyncio.set_event_loop(loop)
@@ -730,7 +732,7 @@ class MyObjectGraph(IObjectGraph):
         :param level: 2 when you are direcly calling. set increased number to show the callee
         :return:
         """
-        from loguru import logger
+        from pinjected.pinjected_logging import logger
         # I need to get the filename and line number of the caller
         if isinstance(target, Designed):
             return self.child_session(target.design)[target.internal_injected]
@@ -739,7 +741,7 @@ class MyObjectGraph(IObjectGraph):
         fn, ln = get_caller_info(level).value_or(("unknown_function", "unknown_line"))
         dep_tree = self.resolver.dependency_tree(target)
         dep_tree = DependencyResolver.unresult_tree(dep_tree)
-        script = self.design.to_vis_graph().to_python_script(
+        script = DIGraph(self.design).to_python_script(
             root=target,
             design_path="__dummy__.design",
         )
@@ -763,7 +765,7 @@ class MyObjectGraph(IObjectGraph):
     def child_session(self, overrides: "Design" = None, trace_logger=None):
         if overrides is None:
             from pinjected import Design
-            overrides = Design()
+            overrides = EmptyDesign
         child_scope = MChildScope(self.scope, set(overrides.keys()),
                                   _trace_logger=trace_logger or self.scope.trace_logger)
         child_graph = MyObjectGraph(None, self.design + overrides, child_scope)
@@ -814,7 +816,7 @@ class AutoSyncGraph:
 def sessioned_value_proxy_context(parent: IObjectGraph, session: IObjectGraph):
     from pinjected.di.dynamic_proxy import DynamicProxyContextImpl
     return DynamicProxyContextImpl(
-        lambda a: a.value,
+        lambda a: a.__value__,
         lambda x: SessionValue(
             parent,
             Designed.bind(Injected.pure(x)),
