@@ -51,56 +51,63 @@ class IProxyContext(Generic[T]):
         """
         raise NotImplementedError()
 
+    def magic_method_impl(self, method_name, tgt: T, *args, **kwargs):
+        return NotImplemented
+        #raise NotImplementedError()
+
 
 @dataclass
 class DelegatedVar(Generic[T]):
     # Generic Var Type that delegates all implementations to cxt.
-    value: T
-    cxt: IProxyContext[T]
+    __value__: T
+    __cxt__: IProxyContext[T]
 
     def __getattr__(self, item):
-        return self.cxt.getattr(self.value, item)
+        return self.__cxt__.getattr(self.__value__, item)
 
     def __call__(self, *args, **kwargs):
-        res = self.cxt.call(self.value, *args, **kwargs)
+        res = self.__cxt__.call(self.__value__, *args, **kwargs)
         return res
 
     def __getitem__(self, key):
-        return self.cxt.getitem(self.value, key)
+        return self.__cxt__.getitem(self.__value__, key)
 
     def eval(self) -> T:
         """this should return the semantic result of calculation."""
-        return self.cxt.eval(self.value)
+        return self.__cxt__.eval(self.__value__)
 
     def __str__(self):
-        return f"{self.cxt.alias_name()}({self.value},{self.cxt})"
+        return f"{self.__cxt__.alias_name()}({self.__value__},{self.__cxt__})"
 
     def __iter__(self):
-        return self.cxt.iter(self.value)
+        return self.__cxt__.iter(self.__value__)
 
     def __getstate__(self):
-        return self.value, self.cxt
+        return self.__value__, self.__cxt__
 
     def __setstate__(self, state):
-        self.value, self.cxt = state
+        self.__value__, self.__cxt__ = state
 
     def __hash__(self):
-        return hash((self.value, self.cxt))
+        return hash((self.__value__, self.__cxt__))
 
     def __add__(self, other):
-        return self.cxt.biop_impl('+', self.value, other)
+        return self.__cxt__.biop_impl('+', self.__value__, other)
 
     def __mul__(self, other):
-        return self.cxt.biop_impl('*', self.value, other)
+        return self.__cxt__.biop_impl('*', self.__value__, other)
 
     def __truediv__(self, other):
-        return self.cxt.biop_impl('/', self.value, other)
+        return self.__cxt__.biop_impl('/', self.__value__, other)
 
     def __mod__(self, other):
-        return self.cxt.biop_impl('%', self.value, other)
+        return self.__cxt__.biop_impl('%', self.__value__, other)
 
     def __eq__(self, other):
-        return self.cxt.biop_impl('==', self.value, other)
+        return self.__cxt__.biop_impl('==', self.__value__, other)
+
+    def __invert__(self):
+        return self.__cxt__.unary_impl('~', self.__value__)
 
     def await__(self):
-        return self.cxt.unary_impl('await', self.value)
+        return self.__cxt__.unary_impl('await', self.__value__)
