@@ -4,7 +4,8 @@ from typing import Any, Callable, Iterator
 from cytoolz import valmap
 
 from pinjected.di.applicative import Applicative
-from pinjected.di.ast import Expr, Call, Attr, GetItem, Object, BiOp, UnaryOp
+from pinjected.di.expr_util import Expr, Call, Attr, GetItem, Object, BiOp, UnaryOp
+from pinjected.di.func_util import fix_args_kwargs
 from pinjected.di.proxiable import T, DelegatedVar, IProxyContext
 
 
@@ -17,8 +18,8 @@ class AstProxyContextImpl(IProxyContext[Expr[T]]):
     def getattr(self, tgt: Expr[T], name: str):
         return self.pure(getattr(tgt, name))
 
-    def call(self, tgt: Expr[T], *args, **kwargs):
-        return self.pure(tgt(*args, **kwargs))
+    def call(self, ___pinjected_tgt___: Expr[T], *args, **kwargs):
+        return self.pure(___pinjected_tgt___(*args, **kwargs))
 
     def pure(self, tgt: Expr[T]) -> "DelegatedVar[T]":
         return DelegatedVar(tgt, self)
@@ -47,50 +48,27 @@ class AstProxyContextImpl(IProxyContext[Expr[T]]):
         return hash(self._alias_name)
 
     def biop_impl(self, op: str, tgt: Expr[T], other):
-        match op:
-            case '+':
-                return self.pure(tgt + other)
-            case '-':
-                return self.pure(tgt - other)
-            case '*':
-                return self.pure(tgt * other)
-            case '/':
-                return self.pure(tgt / other)
-            case '%':
-                return self.pure(tgt % other)
-            case '**':
-                return self.pure(tgt ** other)
-            case '<<':
-                return self.pure(tgt << other)
-            case '>>':
-                return self.pure(tgt >> other)
-            case '&':
-                return self.pure(tgt & other)
-            case '^':
-                return self.pure(tgt ^ other)
-            case '|':
-                return self.pure(tgt | other)
-            case '//':
-                return self.pure(tgt // other)
-            case '@':
-                return self.pure(tgt @ other)
-            case '==':
-                return self.pure(tgt == other)
-            case _:
-                raise NotImplementedError(f"biop {op} not implemented")
+        return self.pure(tgt.biop(op, other))
 
     def unary_impl(self, op: str, tgt: Expr[T]):
-        match op:
-            case 'await':
-                return self.pure(tgt._await_())
-            case _:
-                raise NotImplementedError(f"unary {op} not implemented")
+        return self.pure(tgt.unary(op))
+
+    def magic_method_impl(self, method_name, tgt: Expr[T], *args, **kwargs):
+        return self.getattr(tgt, method_name)(*args, **kwargs)
 
 
 def ast_proxy(tgt, cxt=AstProxyContextImpl(lambda x: x)):
     return DelegatedVar(tgt, cxt)
 
 
+def en_list(thing):
+    return list(thing)
+
+
+def en_tuple(thing):
+    return tuple(thing)
+
+
 def eval_applicative(expr: Expr[T], app: Applicative[T]) -> T:
     def _eval(expr):
         def eval_tuple(expr):
@@ -111,10 +89,10 @@ def eval_applicative(expr: Expr[T], app: Applicative[T]) -> T:
         match expr:
             case Object([*items] as x) if isinstance(x, list):
                 t = app.zip(*[ensure_pure(item) for item in items])
-                return app.map(t, list)
+                return app.map(t, en_list)
             case Object(([*items] as x)) if isinstance(x, tuple):
                 t = app.zip(*[ensure_pure(item) for item in items])
-                return app.map(t, tuple)
+                return app.map(t, en_tuple)
             case Object({**items} as x) if isinstance(x, dict):
                 values = app.zip(*[ensure_pure(item) for item in items.values()])
                 return app.map(values, lambda t: {k: v for k, v in zip(items.keys(), t)})
@@ -130,8 +108,9 @@ def eval_applicative(expr: Expr[T], app: Applicative[T]) -> T:
                 # so all the arguments are converted into Injected if not, then combined together
                 # so if you are to pass an Injected as an argument, you must wrap it with Injected.pure
                 def apply(t):
-                    from loguru import logger
+                    from pinjected.pinjected_logging import logger
                     func, args, kwargs = t
+                    # args,kwargs = fix_args_kwargs(func,args,kwargs)
                     return func(*args, **kwargs)
 
                 applied = app.map(app.zip(injected_func, args, kwargs), apply)
@@ -169,9 +148,12 @@ def eval_applicative(expr: Expr[T], app: Applicative[T]) -> T:
                     app.zip(injected_left, injected_right),
                     eval_biop
                 )
-            case UnaryOp('await',Expr() as tgt):
+            case UnaryOp('await', Expr() as tgt):
                 injected_tgt = _eval(tgt)
                 return app._await_(injected_tgt)
+            case UnaryOp(op, Expr() as tgt):
+                injected_tgt = _eval(tgt)
+                return app.unary(op, injected_tgt)
             case _:
                 raise RuntimeError(f"unsupported ast found!:{type(expr)},{expr}")
 
