@@ -4,7 +4,7 @@ import textwrap
 from pathlib import Path
 from pprint import pformat
 from types import FrameType
-from typing import TypeVar, Dict, Union
+from typing import TypeVar, Dict, Callable, Any
 
 import cloudpickle
 from cytoolz import memoize
@@ -12,13 +12,15 @@ from makefun import create_function
 from returns.maybe import Some
 from returns.result import Failure, Success
 
-from pinjected.di.design import Design
-from pinjected.di.injected import Injected, InjectedPure, InjectedFunction
+from pinjected.di.design import DesignImpl, AddValidation
+from pinjected.di.validation import ValSuccess, ValFailure
+from pinjected.di.injected import Injected, InjectedPure, InjectedFromFunction
 from pinjected.di.metadata.bind_metadata import BindMetadata
 from pinjected.di.metadata.location_data import CodeLocation, ModuleVarLocation
 from pinjected.di.monadic import getitem_opt
 from pinjected.di.proxiable import DelegatedVar
 from pinjected.v2.binds import BindInjected
+from pinjected.v2.keys import StrBindKey, DestructorKey, IBindKey
 
 # is it possible to create a binding class which also has an ability to dynamically add..?
 # yes. actually.
@@ -85,7 +87,7 @@ def check_picklable(tgt: dict):
         failures = rec_val_filter(lambda v: isinstance(v[0], Failure), rec_check)
         # failures = [(k, v, tgt[k]) for k, v in target_check.items() if isinstance(v, Failure)]
 
-        from loguru import logger
+        from pinjected.pinjected_logging import logger
         logger.error(f"Failed to pickle target: {pformat(failures)}")
         logger.error(f"if the error message contains EncodedFile pickling error, "
                      f"check whether the logging module is included in the target object or not.")
@@ -135,9 +137,9 @@ def get_class_aware_args(f):
 
 def to_readable_name(o):
     match o:
-        case BindInjected(InjectedFunction(func, _),_):
+        case BindInjected(InjectedFromFunction(func, _), _):
             return func.__name__
-        case BindInjected(InjectedPure(value),_):
+        case BindInjected(InjectedPure(value), _):
             return value
         case any:
             return any
@@ -180,7 +182,7 @@ def get_dict_diff(a: dict, b: dict):
 
 T = TypeVar("T")
 
-EmptyDesign = Design()
+EmptyDesign = DesignImpl()
 
 
 # mapping is another layer of complication.
@@ -214,7 +216,7 @@ def instances(**kwargs):
 
     Parameters:
     -----------
-    \*\*kwargs : dict
+        kwargs : dict
         Arbitrary keyword arguments representing the instances to be bound.
         Each key represents the name of the dependency, and the value is the corresponding concrete instance or value.
         The function asserts that none of the values are instances of ``DelegatedVar`` or ``Injected``,
@@ -257,7 +259,7 @@ def instances(**kwargs):
         assert not isinstance(v,
                               Injected), f"key {k} is an instance of 'Injected'. passing Injected to 'instances' is forbidden, to prevent human error. use bind_instance instead."
 
-    d = Design().bind_instance(**kwargs)
+    d = DesignImpl().bind_instance(**kwargs)
     return add_code_locations(d, kwargs, inspect.currentframe())
 
 
@@ -271,7 +273,7 @@ def providers(**kwargs):
 
     Parameters:
     -----------
-    \*\*kwargs : dict
+    **kwargs : dict
         Arbitrary keyword arguments. Each key represents the name of the dependency, and the value can be either:
         1. A provider function, which is a callable that returns the necessary object for the dependency. This function can itself require other dependencies.
         2. An `Injected` instance which encapsulates both the creation of the dependency and its own dependencies.
@@ -305,7 +307,15 @@ def providers(**kwargs):
 
     This approach consolidates the registration of dependencies, streamlining the setup process and enhancing the traceability and debuggability of the system components.
     """
-    d = Design().bind_provider(**kwargs)
+    d = DesignImpl().bind_provider(**kwargs)
+    return add_code_locations(d, kwargs, inspect.currentframe())
+
+
+def design(**kwargs):
+    _injecteds = {k: v for k, v in kwargs.items() if isinstance(v, Injected) or isinstance(v, DelegatedVar)}
+    _instances = {k: v for k, v in kwargs.items() if not isinstance(v, Injected) and not isinstance(v, DelegatedVar)}
+
+    d = DesignImpl().bind_instance(**_instances).bind_provider(**_injecteds)
     return add_code_locations(d, kwargs, inspect.currentframe())
 
 
@@ -313,8 +323,9 @@ def add_code_locations(design, kwargs, frame):
     try:
         locs = get_code_locations(list(kwargs.keys()), frame)
         metas = {k: BindMetadata(Some(loc)) for k, loc in locs.items()}
+        # metas = dict()
     except OSError as ose:
-        from loguru import logger
+        from pinjected.pinjected_logging import logger
         logger.warning(f"failed to get code locations:{ose}")
         metas = dict()
 
@@ -388,7 +399,8 @@ def classes(**kwargs):
 
     Parameters:
     -----------
-    \*\*kwargs : dict
+    **kwargs : dict
+
         Arbitrary keyword arguments. Each key is a string that represents the dependency name, and the value is the class to be bound.
 
     Returns:
@@ -412,9 +424,46 @@ def classes(**kwargs):
     -----
     The function automatically adds code locations for the classes, making it easier to track the source of each dependency within the system.
     """
-    d = Design().bind_provider(**kwargs)
+    d = DesignImpl().bind_provider(**kwargs)
     return add_code_locations(d, kwargs, inspect.currentframe())
 
 
+ValidationFunc = Callable[[IBindKey, Any], Any]
+
+
+def validations(**kwargs: ValidationFunc):
+    def get_validator(f):
+        async def impl(key: IBindKey, value):
+            try:
+                maybe_coro = f(key, value)
+                if inspect.iscoroutinefunction(f):
+                    await maybe_coro
+                return ValSuccess()
+            except Exception as e:
+                return ValFailure(e)
+
+        return impl
+
+    validators = dict()
+    for k, f in kwargs.items():
+        key = StrBindKey(k)
+        assert callable(f)
+        validators[key] = get_validator(f)
+    return AddValidation(EmptyDesign, _validations=validators)
+
+
 def injecteds(**kwargs):
-    return Design().bind_provider(**kwargs)
+    return DesignImpl().bind_provider(**kwargs)
+
+
+def destructors(**kwargs):
+    """
+    registers destructors. using DestructorKey.
+    The values must be an async function that takes one argument.
+    """
+    res = instances()
+    for k, v in kwargs.items():
+        tgt = StrBindKey(k)
+        key = DestructorKey(tgt)
+        res += DesignImpl(_bindings={key: DesignImpl.to_bind(v)})
+    return res
