@@ -1,6 +1,8 @@
 from dataclasses import dataclass
 from typing import List, Any
 
+from pinjected.di.validation import ValFailure
+
 
 @dataclass
 class DependencyResolutionFailure:
@@ -15,7 +17,7 @@ class DependencyResolutionFailure:
         return f"failed to find dependency: {self.key} at {self.trace_str()}"
 
     def __repr__(self):
-        return f"DependencyResolutionFailure(key:{self.key}\t,trace:{self.trace_str()},cause: ({self.cause})"
+        return f"DependencyResolutionFailure(key:{self.key},trace:{self.trace_str()},cause: ({self.cause})"
 
 
 class DependencyResolutionError(RuntimeError):
@@ -25,6 +27,21 @@ class DependencyResolutionError(RuntimeError):
             causes = []
         self.causes = causes.copy()
 
+class DependencyValidationError(RuntimeError):
+    def __init__(self, msg: str, cause:ValFailure):
+        super().__init__(msg)
+        self.cause=cause
+
+@dataclass
+class CyclicDependency:
+    key:str
+    trace:List[str]
+    def __repr__(self):
+        trace = self.trace + [self.key]
+        trace_str = " -> ".join(trace)
+        return f"Cyclic Dependency: {trace_str}"
+
+
 
 class _MissingDepsError(RuntimeError):
     def __init__(self, msg: str, name: str, trace: List[str]):
