@@ -5,28 +5,26 @@ from dataclasses import asdict
 from pathlib import Path
 from typing import Mapping
 
-import loguru
-from loguru import logger
+from beartype import beartype
+from pinjected.pinjected_logging import logger
 from returns.maybe import Some
 
 import pinjected
 import pinjected.global_configs
-from pinjected import injected_function, instances, providers, injected_instance, Injected, Design, instance
-from pinjected.di.injected import PartialInjectedFunction, InjectedFunction
+from pinjected import instances, Injected, Design, instance, injected
+from pinjected.di.injected import PartialInjectedFunction, InjectedFromFunction
 from pinjected.di.metadata.location_data import ModuleVarLocation
 from pinjected.graph_inspection import DIGraphHelper
-from pinjected.helper_structure import MetaContext
-from pinjected.helpers import inspect_and_make_configurations, find_default_design_paths
-from pinjected.module_inspector import get_project_root
+from pinjected.helper_structure import MetaContext, IdeaRunConfigurations
+from pinjected.helpers import inspect_and_make_configurations
 from pinjected.module_var_path import ModuleVarPath
-from pinjected.run_config_utils import injected_to_idea_configs
 
 __meta_design__ = instances(
-    default_design_paths=["pinjected.ide_supports.create_configs.my_design"]
+    # ah, this makes my code load my_design
+    default_design_paths=["pinjected.ide_supports.default_design.pinjected_internal_design"]
 )
 
 from pinjected.run_helpers.run_injected import run_injected
-from pinjected.v2.binds import IBind
 from pinjected.v2.keys import IBindKey
 
 
@@ -47,19 +45,22 @@ def run_with_meta_context(
     """
     if not "__meta_design__" in Path(context_module_file_path).read_text():
         raise ValueError(f"{context_module_file_path} does not contain __meta_design__")
-    meta_context = MetaContext.gather_from_path(context_module_file_path)
+    meta_context = MetaContext.gather_from_path(Path(context_module_file_path))
+    default = instances(
+        default_design_paths=[]
+    )
     instance_overrides = instances(
-        module_path=context_module_file_path,
+        module_path=Path(context_module_file_path),
         interpreter_path=sys.executable,
-        meta_context=meta_context
+        meta_context=meta_context,
     ) + instances(**kwargs)
     return run_injected("get", var_path, design_path, return_result=True,
-                        overrides=meta_context.accumulated + instance_overrides,
+                        overrides=default + meta_context.accumulated + instance_overrides,
                         notifier=logger.info
                         )
 
 
-@injected_function
+@injected
 def load_meta_context(
         module_path
 ):
@@ -67,52 +68,37 @@ def load_meta_context(
     return meta_context
 
 
-my_design = instances(
-    logger=loguru.logger,
-    runner_script_path=pinjected.run_config_utils.__file__,
-    custom_idea_config_creator=lambda spec: [],  # type ConfigCreator
-    # this becomes recursive and overflows if we call meta_session inside a parent design...
-    default_design_path=None,
-    print_to_stdout=True
-) + providers(
-    inspect_and_make_configurations=inspect_and_make_configurations,
-    injected_to_idea_configs=injected_to_idea_configs,
-    default_design_paths=lambda module_path, default_design_path: find_default_design_paths(module_path,
-                                                                                            default_design_path),
-    project_root=lambda module_path: Path(get_project_root(module_path)),
-    default_working_dir=lambda project_root: Some(str(project_root)),
-)
-
-
-@injected_function
+@injected
+@beartype
 def create_idea_configurations(
         inspect_and_make_configurations,
-        module_path,
+        module_path: Path,
         print_to_stdout,
         /,
         wrap_output_with_tag=True
 ):
-    pinjected.global_configs.pinjected_TRACK_ORIGIN = False
-    configs = inspect_and_make_configurations(module_path)
-    pinjected.global_configs.pinjected_TRACK_ORIGIN = True
-    logger.info(f"configs:{configs}")
+    with logger.contextualize(tag="create_idea_configurations"):
+        pinjected.global_configs.pinjected_TRACK_ORIGIN = False
+        configs: IdeaRunConfigurations = inspect_and_make_configurations(module_path)
+        pinjected.global_configs.pinjected_TRACK_ORIGIN = True
+        # logger.info(f"configs:{configs}")
 
-    # since stdout is contaminated by many other modules,
-    # We need to think of other way to pass information.
-    # should I use a tempfile?
-    # or, maybe we can use a separator... like <pinjected>...</pinjected>
-    # so that the caller must parse the output.
+        # since stdout is contaminated by many other modules,
+        # We need to think of other way to pass information.
+        # should I use a tempfile?
+        # or, maybe we can use a separator... like <pinjected>...</pinjected>
+        # so that the caller must parse the output.
 
-    if print_to_stdout:
-        data_str = (json.dumps(asdict(configs)))
-        if wrap_output_with_tag:
-            data_str = f"<pinjected>{data_str}</pinjected>"
-        print(data_str)
-    else:
-        return configs
+        if print_to_stdout:
+            data_str = (json.dumps(asdict(configs)))
+            if wrap_output_with_tag:
+                data_str = f"<pinjected>{data_str}</pinjected>"
+            print(data_str)
+        else:
+            return configs
 
 
-@injected_instance
+@instance
 def list_injected_keys(
         default_design_paths: list[str]
 ):
@@ -139,7 +125,7 @@ def get_filtered_signature(func):
     return func.__name__, new_signature_str
 
 
-@injected_instance
+@instance
 def list_completions(
         default_design_paths: list[str]
 ):
@@ -161,7 +147,7 @@ def list_completions(
     def key_to_completion(key):
         tgt = total_mappings[key]
         match tgt:
-            case PartialInjectedFunction(InjectedFunction(object(__original__=func), kw_mapping)):
+            case PartialInjectedFunction(InjectedFromFunction(object(__original__=func), kw_mapping)):
                 name, signature = get_filtered_signature(func)
                 return dict(
                     name=name,
@@ -206,7 +192,7 @@ def design_metadata(
     helper = DIGraphHelper(d)
     metas = []
     for k, bind in helper.total_bindings().items():
-        k:IBindKey
+        k: IBindKey
         match bind.metadata.bind(lambda m: m.code_location):
             case Some(ModuleVarPath(qualified_name)):
                 metas.append(dict(
