@@ -4,8 +4,10 @@ import sys
 from dataclasses import dataclass
 from pathlib import Path
 
+from beartype import beartype
 
-@dataclass
+
+@dataclass(frozen=True)
 class ModuleVarPath:
     """
     represents a path where a variable is defined.
@@ -22,6 +24,9 @@ class ModuleVarPath:
 
     @property
     def module_name(self):
+        split_path = self.path.split(".")
+        if len(split_path) == 1:
+            return split_path[0]
         module = ".".join(self.path.split(".")[:-1])
         return module
 
@@ -33,8 +38,12 @@ class ModuleVarPath:
         return f"from {self.module_name} import {self.var_name}"
 
     @property
-    def module_file_path(self):
-        return sys.modules[self.module_name].__file__
+    def module_file_path(self)->Path:
+        if self.module_name in sys.modules:
+            return Path(sys.modules[self.module_name].__file__)
+        else:
+            __import__(self.module_name)
+            return Path(sys.modules[self.module_name].__file__)
 
     @staticmethod
     def from_local_variable(var_name):
@@ -52,9 +61,9 @@ class ModuleVarPath:
     def depending_import_lines(self):
         return find_import_statements_in_module(self.module_name)
 
-
-def load_variable_by_module_path(full_module_path):
-    from loguru import logger
+@beartype
+def load_variable_by_module_path(full_module_path:str):
+    from pinjected.pinjected_logging import logger
     logger.info(f"loading {full_module_path}")
     module_path_parts = full_module_path.split('.')
     variable_name = module_path_parts[-1]
