@@ -23,8 +23,6 @@ Hmm, an easy way is to add some metadata though..
 Adding feature to an existing data structure is not recommended.
 So I guess I need to introduce a new data structure.
 """
-import loguru
-
 import importlib
 import inspect
 import json
@@ -35,28 +33,27 @@ from pathlib import Path
 from pprint import pformat
 from typing import Optional, List, Dict, OrderedDict, Callable, Any
 
-from loguru import logger
-import fire
+import loguru
 from cytoolz import memoize
+from pinjected.pinjected_logging import logger
 from returns.maybe import Maybe, Some, maybe
 from returns.result import safe, Success, Failure
-from snoop import snoop
 
 from pinjected import Injected, Design, injected_function, Designed
-from pinjected.di.ast import Expr, Call, Object
-from pinjected.di.injected import PartialInjectedFunction, InjectedFunction
+from pinjected.di.expr_util import Expr, Call, Object
+from pinjected.di.injected import PartialInjectedFunction, InjectedFromFunction
 from pinjected.di.proxiable import DelegatedVar
 from pinjected.di.util import instances, providers
+from pinjected.exporter.llm_exporter import add_export_config
 # from pinjected.ide_supports.create_configs import create_idea_configurations
 from pinjected.helper_structure import IdeaRunConfigurations, RunnablePair, IdeaRunConfiguration
-from pinjected.maybe_patch import patch_maybe
-from pinjected.run_helpers.config import ConfigCreationArgs
 from pinjected.helpers import find_default_design_paths
-from pinjected.runnables import get_runnables, RunnableValue
-from pinjected.module_var_path import ModuleVarPath, load_variable_by_module_path
+from pinjected.maybe_patch import patch_maybe
 from pinjected.module_inspector import ModuleVarSpec, inspect_module_for_type
+from pinjected.module_var_path import ModuleVarPath, load_variable_by_module_path
 from pinjected.run_config_utils_v2 import RunInjected
-
+from pinjected.run_helpers.config import ConfigCreationArgs
+from pinjected.runnables import get_runnables, RunnableValue
 
 safe_getattr = safe(getattr)
 
@@ -146,6 +143,7 @@ def injected_to_idea_configs(
         default_working_dir: Maybe[str],
         extract_args_for_runnable,
         logger,
+        internal_idea_config_creator:IdeaConfigCreator,
         custom_idea_config_creator: IdeaConfigCreator,
         /,
         tgt: ModuleVarSpec
@@ -172,7 +170,8 @@ def injected_to_idea_configs(
     results = defaultdict(list)
 
     if not ddps:
-        logger.warning(f"no default design path provided for {tgt.var_path}")
+        logger.warning(f"no default design path provided for {tgt.var_path}, using pinjected.EmptyDesign")
+        ddps.append('pinjected.EmptyDesign')
 
     for ddp in ddps:
         args = extract_args_for_runnable(tgt, ddp, meta)
@@ -201,6 +200,14 @@ def injected_to_idea_configs(
     except Exception as e:
         logger.warning(f"Failed to create custom idea configs for {tgt} because {e}")
         raise RuntimeError(f"Failed to create custom idea configs for {tgt} because {e}") from e
+    try:
+        cfgs = internal_idea_config_creator(tgt)
+        assert cfgs is not None, f"internal_idea_config_creator {internal_idea_config_creator} returned None for {tgt}. return [] if you have no internal configs."
+        for configs in cfgs:
+            results[name].append(configs)
+    except Exception as e:
+        logger.warning(f"Failed to create internal idea configs for {tgt} because {e}")
+        raise RuntimeError(f"Failed to create internal idea configs for {tgt} because {e}") from e
     return IdeaRunConfigurations(configs=results)
 
 
@@ -265,7 +272,7 @@ default_design = design = providers(
     #    str(project_root)),
     # default_design_path=lambda default_design_paths: default_design_paths[0]
 ) + instances(
-    logger=loguru.logger,
+    logger=logger,
     custom_idea_config_creator=lambda x: [],  # type ConfigCreator
     # meta_context=meta_context,
     # module_path=self.module_path,
@@ -303,7 +310,7 @@ def extract_extra_codes(ast: Expr) -> ModuleVarPath:
     # TODO include the module path in the Inejcted Function
     match ast:
         case Call(
-            Object(InjectedFunction(object(__original_code__=code, __name__=name, __module__=mod), _args)),
+            Object(InjectedFromFunction(object(__original_code__=code, __name__=name, __module__=mod), _args)),
             args,
             kwargs):
             mvp = ModuleVarPath(f"{mod}.{name}")
@@ -332,7 +339,7 @@ def make_sandbox_extra(tgt: ModuleVarSpec):
 {impl}
 {usage}
 """
-        case PartialInjectedFunction(InjectedFunction(object(__original_code__=usage), args)):
+        case PartialInjectedFunction(InjectedFromFunction(object(__original_code__=usage), args)):
             return f"""
 {usage}
 """
@@ -469,7 +476,7 @@ def main_override_resolver(query) -> Design:
     if isinstance(query, dict):
         return instances(**query)
     elif query is None:
-        return Design()
+        return EmptyDesign
     elif query.endswith('.json'):
         import json
         if not Path(query).exists():
@@ -523,7 +530,7 @@ def run_main():
     """
     import inspect
     import fire
-    from loguru import logger
+    from pinjected.pinjected_logging import logger
     module_path = provide_module_path(logger, inspect.currentframe().f_back)
     cfg = ConfigCreationArgs(
         module_path=module_path,
@@ -570,7 +577,7 @@ def run_idea_conf(conf: IdeaRunConfiguration, *args, **kwargs):
 
 @memoize
 def get_designs_from_module(module_path: Path):
-    from loguru import logger
+    from pinjected.pinjected_logging import logger
     logger.info(f"trying to import designs from {module_path}")
 
     def accept(name, x):
@@ -599,4 +606,6 @@ if __name__ == '__main__':
 
 __meta_design__ = instances(
     default_design_paths=["pinjected.run_config_utils.__meta_design__"]
+) + providers(
+    internal_idea_config_creator=add_export_config
 )
