@@ -3,26 +3,26 @@ import platform
 import uuid
 from collections import defaultdict
 from dataclasses import dataclass
-from itertools import chain
-from typing import Callable, List, Dict, Union
+from pathlib import Path
+from typing import Callable, List, Union
 
 import networkx as nx
 from cytoolz import memoize
-from loguru import logger
+from pinjected.pinjected_logging import logger
 from returns.pipeline import is_successful
-from returns.result import safe, Result, Failure
+from returns.result import safe, Failure
 
-from pinjected.di.graph import providable_to_injected
-from pinjected.providable import Providable
-from pinjected.di.injected import Injected, InjectedFunction, InjectedPure, MappedInjected, \
+from pinjected.di.app_injected import EvaledInjected
+from pinjected.di.expr_util import show_expr
+from pinjected.di.injected import Injected, InjectedFromFunction, InjectedPure, MappedInjected, \
     ZippedInjected, MZippedInjected, InjectedByName, extract_dependency, InjectedWithDefaultDesign, \
     PartialInjectedFunction
 from pinjected.di.proxiable import DelegatedVar
-from pinjected import Design
-from pinjected.exceptions import DependencyResolutionFailure, _MissingDepsError
+from pinjected.exceptions import DependencyResolutionFailure, _MissingDepsError, CyclicDependency
 from pinjected.graph_inspection import DIGraphHelper
 from pinjected.module_var_path import ModuleVarPath
 from pinjected.nx_graph_util import NxGraphUtil
+from pinjected.providable import Providable
 from pinjected.v2.binds import BindInjected
 
 
@@ -55,16 +55,19 @@ def get_color(n_edges):
     return rgb_to_hex(tuple(rgb))
 
 
+class MissingKeyError(RuntimeError):
+    pass
+
 @dataclass
 class DIGraph:
-    src: Design
+    src: "Design"
 
     def new_name(self, base: str):
         return f"{base}_{str(uuid.uuid4())[:6]}"
 
     def __post_init__(self):
         self.helper = DIGraphHelper(self.src)
-        self.explicit_mappings:dict[str,Injected] = self.helper.total_mappings()
+        self.explicit_mappings: dict[str, Injected] = self.helper.total_mappings()
 
         self.direct_injected = dict()
         self.injected_to_id = dict()
@@ -80,7 +83,7 @@ class DIGraph:
                 di = self.direct_injected[src]
                 return self.resolve_injected(di)
             else:
-                raise RuntimeError(f"key not found!:{src}")
+                raise MissingKeyError(f"DI key not found!:{src}")
 
         self.deps_impl = deps_impl
 
@@ -93,7 +96,7 @@ class DIGraph:
         #        else:
         #            assert False
         _id = self.injected_to_id[i]
-        if isinstance(i, InjectedFunction):
+        if isinstance(i, InjectedFromFunction):
             deps = []
             for k, v in i.kwargs_mapping.items():
                 dep_name = f"{k}_{_id}"
@@ -102,10 +105,10 @@ class DIGraph:
                         injected = Injected.by_name(v)
                     case type():
                         injected = Injected.bind(v)
-                    case f if callable(f):
-                        injected = Injected.bind(f)
                     case Injected():
                         injected = v
+                    case f if callable(f):
+                        injected = Injected.bind(f)
                 self.direct_injected[dep_name] = injected
 
                 # logger.info(f"add direct injected node:\n{tabulate.tabulate(list(self.direct_injected.items()))}")
@@ -190,29 +193,36 @@ class DIGraph:
 
     def di_dfs_validation(self, src):
         def dfs(node: str, trace=[]):
+            # logger.info(f"dfs:{node},{trace}")
             nexts = []
             try:
                 nexts: List[str] = self.dependencies_of(node)
-            except Exception as e:
+            except MissingKeyError as e:
                 yield DependencyResolutionFailure(node, trace, e)
             for n in nexts:
-                yield from dfs(n, trace + [n])
+                if n in trace:
+                    yield CyclicDependency(n, trace)
+                else:
+                    yield from dfs(n, trace + [n])
 
         yield from dfs(src, [src])
 
-    def distilled(self,tgt:Providable)->Design:
+    def distilled(self, tgt: Providable) -> "Design":
+        from pinjected import providers
+        from pinjected import Design
         match tgt:
             case str():
                 deps = set([t[1] for t in self.di_dfs(tgt)])
                 nodes = set([t[0] for t in self.di_dfs(tgt)])
-                distilled = Design(
+                distilled = Design.from_bindings(
                     {k: self.src[k] for k in deps | nodes}
                 )
                 return distilled
 
             case _:
+                from pinjected.di.graph import providable_to_injected
                 _injected = providable_to_injected(tgt)
-                tmp_design = self.src.bind_provider(
+                tmp_design = self.src + providers(
                     __target__=_injected
                 )
                 return tmp_design.to_vis_graph().distilled("__target__")
@@ -231,12 +241,11 @@ class DIGraph:
             res = file + "\n" + src
 
         except Exception as e:
-            from loguru import logger
+            from pinjected.pinjected_logging import logger
             logger.warning(f"{repr(e)[:100]} error from {repr(f)[:100]}")
             res = f"failed:{f} from {f}"
         return res
 
-
     @staticmethod
     def get_source_repr(f):
         res = DIGraph.get_source(f)
@@ -251,13 +260,17 @@ class DIGraph:
         match tgt:
             case InjectedWithDefaultDesign(src, default_design):
                 return self.parse_injected(src)
-            case InjectedFunction(f):
-                desc = f"Injected:{safe(getattr)(f, '__name__').value_or(repr(f))}"
+            case InjectedFromFunction(f) as _if:
+                try:
+                    desc = f"Injected:{safe(getattr)(_if.original_function, '__name__').value_or(repr(f))}"
+                except Exception as e:
+                    logger.error(f"failed to get func info from {f} due to {e}")
+                    raise e
                 return ("injected", desc, self.get_source_repr(f))
             case InjectedPure(v):
                 desc = f"Pure:{v}"
                 return ("injected", desc, self.get_source_repr(v) if isinstance(v, Callable) else str(v))
-            case PartialInjectedFunction(InjectedFunction(src)):
+            case PartialInjectedFunction(InjectedFromFunction(src)):
                 desc = f"partial=>{src.__name__}"
                 return ("injected", desc, self.get_source_repr(src))
             case PartialInjectedFunction(src):
@@ -275,6 +288,10 @@ class DIGraph:
             case InjectedByName(name):
                 desc = f"{name}"
                 return ("injected", desc, "by_name")
+            case EvaledInjected(val, ast):
+                expr = show_expr(ast)
+                desc = f"Eval({expr})"
+                return ("injected", desc, expr)
             case Injected() as injected:
                 desc = f"{injected.__class__.__name__}"
                 return ("injected", desc, str(injected))
@@ -284,7 +301,8 @@ class DIGraph:
     def create_dependency_digraph_rooted(self, root: Injected, root_name="__root__",
                                          replace_missing=True
                                          ) -> NxGraphUtil:
-        tmp_design = self.src.bind_provider(**{root_name: root})
+        from pinjected import providers
+        tmp_design = self.src + providers(**{root_name: root})
         return DIGraph(tmp_design) \
             .create_dependency_digraph(
             root_name,
@@ -292,7 +310,7 @@ class DIGraph:
         )
 
     def create_graph_from_nodes(self, nodes: List[str], replace_missing=True):
-        from loguru import logger
+        from pinjected.pinjected_logging import logger
         logger.info(f"making dependency graph for {nodes}.")
         nx_graph = nx.DiGraph()
         # why am I seeing no deps?
@@ -310,7 +328,7 @@ class DIGraph:
                 match tgt:
                     case Injected():
                         return self.parse_injected(tgt)
-                    case BindInjected(tgt,_):
+                    case BindInjected(tgt, _):
                         return self.parse_injected(tgt)
                     case cls if isinstance(cls, type):
                         return ("class", cls.__name__, self.get_source_repr(cls))
@@ -418,7 +436,7 @@ g = d.to_graph()
         if isinstance(roots, str):
             roots = [roots]
         # hmm,
-        from loguru import logger
+        from pinjected.pinjected_logging import logger
         logger.info(f"making dependency graph for {roots}.")
         nx_graph = nx.DiGraph()
         # why am I seeing no deps?
@@ -463,16 +481,19 @@ g = d.to_graph()
         nt.prep_notebook()
         return nt.show("__notebook__.html")
 
-    def save_as_html(self, tgt: Injected, name: str, visualize_missing=True, show=True):
+    def save_as_html(self, tgt: Injected, dst_root: Path, visualize_missing=True, show=True):
         nx = self.create_dependency_digraph_rooted(tgt, replace_missing=visualize_missing)
-        nx.save_as_html(name, show=show)
+        #nx.save_as_html(name, show=show)
+        return nx.save_as_html_at(dst_root)
+
+
 
     def plot(self, roots: Union[str, List[str]], visualize_missing=True):
         if "darwin" in platform.system().lower():
             G = self.create_dependency_digraph(roots, replace_missing=visualize_missing)
             G.plot_mpl()
         else:
-            from loguru import logger
+            from pinjected.pinjected_logging import logger
             logger.warning("visualization of a design is disabled for non mac os.")
 
     def show_html(self, roots, visualize_missing=True):
@@ -482,7 +503,6 @@ g = d.to_graph()
     def show_injected_html(self, tgt: Injected, name: str = None):
         tgt = Injected.ensure_injected(tgt)
         assert isinstance(tgt, Injected)
-        assert platform.system().lower() == "darwin"
         nx_graph = self.create_dependency_digraph_rooted(tgt, name or "__root__", replace_missing=True)
         nx_graph.show_html_temp()
 
@@ -491,9 +511,9 @@ g = d.to_graph()
         self.create_dependency_digraph(roots, replace_missing=True, root_group=None).show_html_temp()
 
 
-
-def create_dependency_graph(d: Design, roots: List[str], output_file="dependencies.html"):
-    dig = DIGraph(d.bind_instance(
+def create_dependency_graph(d: "Design", roots: List[str], output_file="dependencies.html"):
+    from pinjected import instances
+    dig = DIGraph(d + instances(
         job_type="net_visualization"
     ))
     nt = dig.create_dependency_network(roots)
